@@ -23,6 +23,8 @@
 #include <config.h>
 
 #include <gtk/gtk.h>
+#include <glib.h>
+#include <glib/gprintf.h>
 #include <glib/gi18n.h>
 #include <string.h>
 
@@ -37,7 +39,7 @@ create_model (void)
 {
 	GtkTreeStore *mdl = gtk_tree_store_new (NUM_TREE_COLUMNS,
 						G_TYPE_STRING,	/* COL_DIR_NAME */
-						G_TYPE_STRING,	/* COL_H_FULLPATH */
+						G_TYPE_STRING,	/* COL_H_PARSENAME */
 						G_TYPE_DOUBLE,	/* COL_H_PERC */
 						G_TYPE_STRING,	/* COL_DIR_SIZE */
 						G_TYPE_UINT64,	/* COL_H_SIZE */
@@ -63,32 +65,16 @@ on_tv_row_expanded (GtkTreeView *treeview,
 static void
 on_tv_cur_changed (GtkTreeView *treeview, gpointer data)
 {
-
 	GtkTreeIter iter;
-	gchar *text = NULL;
+	gchar *parsename = NULL;
 
 	gtk_tree_selection_get_selected (gtk_tree_view_get_selection (treeview), NULL, &iter);
 
 	if (gtk_tree_store_iter_is_valid (baobab.model, &iter)) {
-			gtk_tree_model_get (GTK_TREE_MODEL (baobab.model), &iter,
-				    COL_H_FULLPATH, &text, -1);
+		gtk_tree_model_get (GTK_TREE_MODEL (baobab.model), &iter,
+				    COL_H_PARSENAME, &parsename, -1);
 	}
-	
-	set_glade_widget_sens("menu_treemap",FALSE);
-	if (text) {
-		gchar *msg;
-
-		/* make sure it is utf8 */
-		msg = g_filename_display_name (text);
-
-		set_statusbar (msg);
-		if (strcmp (text, "") != 0 )
-			set_glade_widget_sens("menu_treemap",TRUE);
-
-		g_free (msg);
-		g_free (text);
 	}
-}
 
 static gboolean
 on_tv_button_press (GtkWidget *widget,
@@ -97,8 +83,7 @@ on_tv_button_press (GtkWidget *widget,
 {
 	GtkTreePath *path;
 	GtkTreeIter iter;
-	gchar *trash_path, *dir_path;
-	gboolean is_trash = FALSE;
+	
 
 	if (baobab.CONTENTS_CHANGED_DELAYED) {
 		baobab.CONTENTS_CHANGED_DELAYED = FALSE;
@@ -122,26 +107,19 @@ on_tv_button_press (GtkWidget *widget,
 	gtk_tree_model_get_iter (GTK_TREE_MODEL (baobab.model), &iter,
 				 path);
 	gtk_tree_model_get (GTK_TREE_MODEL (baobab.model), &iter,
-			    COL_H_FULLPATH, &baobab.selected_path, -1);
+			    COL_H_PARSENAME, &baobab.selected_path, -1);
 	
-	if (strcmp (baobab.selected_path, "") == 0) {
-		set_glade_widget_sens("menu_treemap",FALSE);
-		gtk_tree_path_free (path);
-		return FALSE;
-	}
-
 	/* right-click */
 	if (event->button == 3) {
-		trash_path = get_trash_path(baobab.selected_path);
-		dir_path = g_path_get_dirname(baobab.selected_path);
-		if (trash_path)
-			if (strcmp(trash_path, dir_path)==0)
-				is_trash = TRUE;
-		popupmenu_list (path, event, is_trash);
+		GFile *file;
 
+		file = g_file_parse_name (baobab.selected_path);
+		if (g_file_query_exists (file, NULL)) {
+		     popupmenu_list (path, event, can_trash_file (file));
+		}
 		gtk_tree_path_free (path);
-		g_free(trash_path);
-		g_free(dir_path);
+		g_object_unref (file);
+
 		return FALSE;
 	}
 
